# app/api/v1/endpoints/auth.py
import logging
from fastapi import APIRouter, Depends
from fastapi.security import OAuth2PasswordRequestForm
from sqlalchemy.ext.asyncio import AsyncSession

from app.database import get_db
from app.schemas.user import UserCreate, UserPublic
from app.schemas.auth import Token
from app.crud import user as crud_user
from app.core.security import verify_password, create_access_token
from app.core.exceptions import (
    EmailAlreadyRegisteredError,
    InvalidCredentialsError,
    UserCreationError
)

logger = logging.getLogger(__name__)
router = APIRouter()

@router.post(
    "/signup",
    response_model=UserPublic,
    status_code=201,
    summary="Register New User",
    description="Creates a new user account.",
    tags=["Authentication"]
)
async def signup(
    user_in: UserCreate,
    db: AsyncSession = Depends(get_db)
):
    """
    Handles user registration.
    - Validates input data.
    - Checks if email already exists.
    - Hashes the password.
    - Stores the new user in the database.
    """
    logger.info(f"Signup attempt for email: {user_in.email}")
    existing_user = await crud_user.get_user_by_email(db, email=user_in.email)
    if existing_user:
        logger.warning(f"Signup failed: Email already registered - {user_in.email}")
        raise EmailAlreadyRegisteredError()

    try:
        created_user = await crud_user.create_user(db=db, user_in=user_in)
        logger.info(f"User created successfully: {created_user.email} (ID: {created_user.id})")
        return created_user
    except Exception as e:
        logger.error(f"Error during user creation for {user_in.email}: {e}", exc_info=True)
        raise UserCreationError()

@router.post(
    "/login",
    response_model=Token,
    summary="User Login",
    description="Authenticates a user and returns an access token.",
    tags=["Authentication"]
)
async def login(
    form_data: OAuth2PasswordRequestForm = Depends(),
    db: AsyncSession = Depends(get_db)
):
    """
    Handles user login.
    - Finds user by email (provided in 'username' field of form).
    - Verifies the provided password against the stored hash.
    - Generates and returns a JWT access token upon successful authentication.
    """
    logger.info(f"Login attempt for user: {form_data.username}")
    user = await crud_user.get_user_by_email(db, email=form_data.username)

    # Check if user exists and password is correct
    if not user or not verify_password(form_data.password, user.password_hash):
        logger.warning(f"Login failed: Invalid credentials for user {form_data.username}")
        raise InvalidCredentialsError()

    # Generate JWT
    access_token = create_access_token(subject=user.email)
    logger.info(f"Login successful, token generated for user: {user.email}")
    return Token(access_token=access_token, token_type="bearer")