// src/lib/apiClient.ts

// Import necessary modules/types
import { browser } from '$app/environment'; // For checks if needed
import { error } from '@sveltejs/kit'; // Can be used for throwing errors in load functions
import { authStore, logout, getCurrentToken } from './stores/authStore'; // Import store and helpers

// --- Configuration ---
// Read base URL from Vite environment variables
// Ensure VITE_API_BASE_URL is set in your fe/.env file (e.g., VITE_API_BASE_URL=http://localhost:8000/api)
const BASE_URL = import.meta.env.VITE_API_BASE_URL;

// Initial check for configuration during module load (optional but good practice)
if (!BASE_URL && browser) { // Only log error in browser, avoid during SSR build if possible
    console.error(
        'VITE_API_BASE_URL is not defined. Please set it in your .env file. API calls may fail.'
    );
}

// --- Custom Error Class for API Client ---
export class ApiClientError extends Error {
    status: number; // HTTP status code
    errorData: unknown; // Parsed error data from response body (if any)

    constructor(message: string, status: number, errorData: unknown = null) {
        super(message); // Pass message to the base Error class
        this.name = 'ApiClientError'; // Custom error name
        this.status = status;
        this.errorData = errorData;

        // Attempt to capture a cleaner stack trace in V8 environments (Node, Chrome)
        // Conditionally check if the non-standard captureStackTrace exists
        if (typeof (Error as any).captureStackTrace === 'function') {
            // Call it if it exists, casting Error to 'any' to bypass static type check
            (Error as any).captureStackTrace(this, ApiClientError); // Pass 'this' and the constructor
        }
    }
}

// --- Request Options Interface ---
// Extends standard RequestInit but omits 'body' as we handle it separately
interface RequestOptions extends Omit<RequestInit, 'body' | 'headers'> {
    headers?: HeadersInit;
}


// --- Core Request Function ---
// Uses generics <T> to allow specifying the expected successful response data type
async function request<T = unknown>(
    method: string,
    path: string,
    bodyData?: unknown,
    options: RequestOptions = {}
): Promise<T> {
    if (!BASE_URL) {
        throw new Error('API Base URL (VITE_API_BASE_URL) is not configured.');
    }

    const cleanBase = BASE_URL.replace(/\/$/, '');
    const cleanPath = path.replace(/^\//, '');
    const url = `${cleanBase}/${cleanPath}`;

    // --- Refined Header Handling ---
    const headers = new Headers({ Accept: 'application/json' });

    if (options.headers) {
        new Headers(options.headers).forEach((value, key) => {
            headers.set(key, value);
        });
    }

    // --- Prepare Request Body and Set Content-Type ---
    let processedBody: BodyInit | null = null;
    if (bodyData !== undefined && bodyData !== null) {
        if (bodyData instanceof URLSearchParams) {
            headers.set('Content-Type', 'application/x-www-form-urlencoded');
            processedBody = bodyData;
        } else if (bodyData instanceof FormData) {
            processedBody = bodyData;
        } else if (typeof bodyData === 'object') {
            headers.set('Content-Type', 'application/json');
            try { processedBody = JSON.stringify(bodyData); }
            catch (e) { throw new Error("Invalid JSON body data provided."); }
        } else {
            headers.set('Content-Type', 'application/json');
            try { processedBody = JSON.stringify(bodyData); }
            catch (e) { throw new Error("Invalid body data provided."); }
        }
    }

    // --- Add Authorization Header ---
    const currentToken = getCurrentToken();
    if (currentToken && !headers.has('Authorization')) {
        headers.set('Authorization', `Bearer ${currentToken}`);
    }

    // --- Assemble fetch options carefully ---
    const fetchOptions: RequestInit = {
        method: method.toUpperCase(),
        headers: headers,
        body: processedBody,
    };

    const { headers: _, ...restOfOptions } = options;
    Object.assign(fetchOptions, restOfOptions);

    fetchOptions.credentials = fetchOptions.credentials ?? 'same-origin';
    fetchOptions.mode = fetchOptions.mode ?? 'cors';
    fetchOptions.cache = fetchOptions.cache ?? 'default';

    // --- Execute Fetch and Handle Response ---
    try {
        const response = await fetch(url, fetchOptions);
        if (!response.ok) {
            let errorJson: unknown = null;
            try { errorJson = await response.json(); }
            catch (e) { /* ignore */ }
            const errorToThrow = new ApiClientError(`HTTP Error ${response.status}`, response.status, errorJson);
            if (response.status === 401) { logout(); }
            throw errorToThrow;
        }
        if (response.status === 204) { return null as T; }
        return (await response.json()) as T;
    } catch (err) {
        if (err instanceof ApiClientError && err.status === 401) { logout(); }
        if (err instanceof ApiClientError) { throw err; }
        throw new ApiClientError('Unknown error occurred', 0, err);
    }
}


// --- Convenience Methods (GET, POST, PUT, DELETE, PATCH) ---
// Provide simple wrappers around the core 'request' function

export const apiClient = {
    /**
     * Performs a GET request.
     * @template T The expected type of the response data.
     * @param path API endpoint path (e.g., '/v1/users/me').
     * @param options Optional fetch request options.
     * @returns Promise resolving to the parsed JSON response body of type T.
     */
    get: <T = unknown>(path: string, options: RequestOptions = {}): Promise<T> => {
        return request<T>('GET', path, undefined, options);
    },

    /**
     * Performs a POST request.
     * @template T The expected type of the response data.
     * @param path API endpoint path (e.g., '/v1/auth/signup').
     * @param data Request body data (object, FormData, URLSearchParams).
     * @param options Optional fetch request options.
     * @returns Promise resolving to the parsed JSON response body of type T.
     */
    post: <T = unknown>(path: string, data: unknown, options: RequestOptions = {}): Promise<T> => {
        return request<T>('POST', path, data, options);
    },

    /**
     * Performs a PUT request.
     * @template T The expected type of the response data.
     * @param path API endpoint path.
     * @param data Request body data.
     * @param options Optional fetch request options.
     * @returns Promise resolving to the parsed JSON response body of type T.
     */
    put: <T = unknown>(path: string, data: unknown, options: RequestOptions = {}): Promise<T> => {
        return request<T>('PUT', path, data, options);
    },

    /**
     * Performs a DELETE request.
     * @template T The expected type of the response data (often null or void).
     * @param path API endpoint path.
     * @param options Optional fetch request options.
     * @returns Promise resolving to the parsed JSON response body (often null for 204).
     */
    delete: <T = unknown>(path: string, options: RequestOptions = {}): Promise<T> => {
        // DELETE requests might or might not have a body depending on API design
        return request<T>('DELETE', path, undefined, options);
    },

    /**
     * Performs a PATCH request.
     * @template T The expected type of the response data.
     * @param path API endpoint path.
     * @param data Request body data (usually partial updates).
     * @param options Optional fetch request options.
     * @returns Promise resolving to the parsed JSON response body of type T.
     */
    patch: <T = unknown>(path: string, data: unknown, options: RequestOptions = {}): Promise<T> => {
        return request<T>('PATCH', path, data, options);
    }
};

// Optional: Export the error class as well if needed externally
// export { ApiClientError };