// src/lib/stores/authStore.ts
import { writable, get } from 'svelte/store';
import { browser } from '$app/environment'; // Import browser check

// --- Define Types ---

// You should ideally have a shared UserPublic type or define it here
// matching the backend UserPublic schema
interface UserPublic {
    id: number;
    email: string;
    name?: string | null;
    created_at: string; // Date might be string in JSON
}

interface AuthState {
    isAuthenticated: boolean;
    user: UserPublic | null;
    token: string | null;
}

// --- Store Initialization ---

const AUTH_TOKEN_KEY = 'authToken'; // Key for localStorage

const initialAuthState: AuthState = {
    isAuthenticated: false,
    user: null,
    token: null
};

// Create the writable store
export const authStore = writable<AuthState>(initialAuthState);

// --- Persistence Logic ---

// Load initial state from localStorage (only in browser)
if (browser) {
    const storedToken = localStorage.getItem(AUTH_TOKEN_KEY);
    if (storedToken) {
        // Token exists, tentatively set state.
        // We don't know if it's *valid* yet, nor do we have user data.
        // A call to /users/me on app load could validate & fetch user data.
        authStore.update((state) => ({
            ...state,
            token: storedToken,
            // Keep isAuthenticated false until token is validated/user fetched
            // Or set to true tentatively if you prefer optimistic UI
            isAuthenticated: true // Optimistic: assume token might be valid
        }));
        console.log('AuthStore: Loaded token from localStorage.');
    }
}

// Subscribe to store changes to persist the token (only in browser)
authStore.subscribe((state) => {
    if (browser) {
        if (state.token) {
            // Save token to localStorage when it exists
            localStorage.setItem(AUTH_TOKEN_KEY, state.token);
            console.log('AuthStore: Token saved to localStorage.');
        } else {
            // Remove token from localStorage when it's null (logout)
            localStorage.removeItem(AUTH_TOKEN_KEY);
            console.log('AuthStore: Token removed from localStorage.');
        }
    }
});


// --- Action Functions ---

/**
 * Updates the auth store upon successful login.
 * @param token The JWT access token.
 * @param userData The public user data received from the login/signup or /users/me endpoint.
 */
export function login(token: string, userData: UserPublic): void {
    authStore.set({
        isAuthenticated: true,
        user: userData,
        token: token
    });
    console.log('AuthStore: User logged in.', userData);
}

/**
 * Resets the auth store to its initial state (logged out).
 */
export function logout(): void {
    authStore.set(initialAuthState);
    console.log('AuthStore: User logged out.');
}

/**
 * Updates only the user information in the store, keeping auth state.
 * Useful after fetching fresh user data from /users/me.
 * @param userData The updated public user data.
 */
export function updateUser(userData: UserPublic): void {
    authStore.update(state => {
        if (state.isAuthenticated) {
            return { ...state, user: userData };
        }
        return state; // No change if not authenticated
    });
    console.log('AuthStore: User data updated.', userData);
}


// --- Helper to get token synchronously (use with caution) ---
/**
 * Gets the current token synchronously from the store.
 * Primarily intended for use within the apiClient where reactivity isn't needed.
 * @returns The current token string or null.
 */
export function getCurrentToken(): string | null {
    return get(authStore).token;
}