// src/routes/(app)/dashboard/+page.ts
import { error } from '@sveltejs/kit';
import { apiClient, ApiClientError } from '$lib/apiClient';
import type { GroupPublic } from '$lib/schemas/group'; // Import the Group type
import type { PageLoad } from './$types'; // SvelteKit's type for load functions
import type { ListPublic } from '$lib/schemas/list';

// Define the expected shape of the data returned by this load function
export interface DashboardLoadData {
    groups: GroupPublic[];
    error?: string | null; // Optional error message property
}

export const load: PageLoad<DashboardLoadData> = async ({ fetch }) => {
    // Note: SvelteKit's 'fetch' is recommended inside load functions
    // as it handles credentials and relative paths better during SSR/CSR.
    // However, our apiClient uses the global fetch but includes auth logic.
    // For consistency, we can continue using apiClient here.
    console.log('Dashboard page load: Fetching groups...');
    try {
        const groups = await apiClient.get<GroupPublic[]>('/v1/groups'); // apiClient adds auth header
        const lists = await apiClient.get<ListPublic[]>('/v1/lists'); // apiClient adds auth header
        console.log('Dashboard page load: Groups fetched successfully', groups);
        return {
            groups: groups ?? [], // Return empty array if API returns null/undefined
            lists: lists ?? [],
            error: null
        };
    } catch (err) {
        console.error('Dashboard page load: Failed to fetch groups:', err);
        let errorMessage = 'Failed to load groups.';
        if (err instanceof ApiClientError) {
            // Specific API error handling (authStore's 401 handling should have run)
            errorMessage = `Failed to load groups (Status: ${err.status}). Please try again later.`;
            // If it was a 401, the layout guard should ideally redirect before this load runs,
            // but handle defensively.
            if (err.status === 401) {
                errorMessage = "Your session may have expired. Please log in again."
                // Redirect could also happen here, but layout guard is primary place
                // throw redirect(307, '/login?sessionExpired=true');
            }
        } else if (err instanceof Error) {
            errorMessage = `Network or client error: ${err.message}`;
        }
        // Return empty list and the error message
        return {
            groups: [],
            error: errorMessage
        };
    }
};