// src/routes/(app)/lists/[listId]/+page.ts
import { error } from '@sveltejs/kit';
import { apiClient, ApiClientError } from '$lib/apiClient';
import type { ListDetail } from '$lib/schemas/list';
// --- Use the correct generated type ---
import type { PageLoad } from './$types'; // This type includes correctly typed 'params'

export interface ListDetailPageLoadData {
    list: ListDetail;
}

export const load: PageLoad<ListDetailPageLoadData> = async ({ params, fetch }) => {
    const listId = params.listId;
    console.log(`List Detail page load: Fetching data for list ID: ${listId}`);

    if (!listId || isNaN(parseInt(listId, 10))) {
        throw error(400, 'Invalid List ID');
    }
    try {
        // Fetch the specific list details (expecting items to be included)
        // The backend GET /api/v1/lists/{list_id} should return ListDetail schema
        const listData = await apiClient.get<ListDetail>(`/v1/lists/${listId}`);

        if (!listData) {
            // Should not happen if API call was successful, but check defensively
            throw error(404, 'List not found (API returned no data)');
        }

        console.log('List Detail page load: Data fetched successfully', listData);
        return {
            list: listData
        };
    } catch (err) {
        console.error(`List Detail page load: Failed to fetch list ${listId}:`, err);
        if (err instanceof ApiClientError) {
            if (err.status === 404) {
                throw error(404, 'List not found');
            }
            if (err.status === 403) {
                // User is authenticated (layout guard passed) but not member/creator
                throw error(403, 'Forbidden: You do not have permission to view this list');
            }
            // For other API errors (like 500)
            throw error(err.status || 500, `API Error: ${err.message}`);
        } else if (err instanceof Error) {
            // Network or other client errors
            throw error(500, `Failed to load list data: ${err.message}`);
        } else {
            // Unknown error
            throw error(500, 'An unexpected error occurred while loading list data.');
        }
    }
};