// src/routes/(app)/lists/[listId]/edit/+page.ts
import { error } from '@sveltejs/kit';
import { apiClient, ApiClientError } from '$lib/apiClient';
import type { GroupPublic } from '$lib/schemas/group';
import type { ListPublic } from '$lib/schemas/list'; // Use ListPublic or ListDetail
import type { PageLoad } from './$types';

export interface EditListPageLoadData {
    list: ListPublic; // Or ListDetail if needed
    groups: GroupPublic[];
    error?: string | null; // For group loading errors
}

// Fetch the specific list to edit AND the user's groups for the dropdown
export const load: PageLoad<EditListPageLoadData> = async ({ params, fetch }) => {
    const listId = params.listId;
    console.log(`Edit List page load: Fetching list ${listId} and groups...`);

    if (!listId || isNaN(parseInt(listId, 10))) {
        throw error(400, 'Invalid List ID');
    }

    try {
        // Fetch list details and groups in parallel
        // Use apiClient for automatic auth handling
        const [listResult, groupsResult] = await Promise.allSettled([
            apiClient.get<ListPublic>(`/v1/lists/${listId}`), // Fetch specific list
            apiClient.get<GroupPublic[]>('/v1/groups') // Fetch groups for dropdown
        ]);

        let listData: ListPublic;
        let groupsData: GroupPublic[] = [];
        let groupsError: string | null = null;

        // Process list result
        if (listResult.status === 'fulfilled' && listResult.value) {
            listData = listResult.value;
        } else {
            // Handle list fetch failure
            const reason = listResult.status === 'rejected' ? listResult.reason : new Error('List data missing');
            console.error(`Edit List page load: Failed to fetch list ${listId}:`, reason);
            if (reason instanceof ApiClientError) {
                if (reason.status === 404) throw error(404, 'List not found');
                if (reason.status === 403) throw error(403, 'Forbidden: You cannot edit this list');
                throw error(reason.status || 500, `API Error loading list: ${reason.message}`);
            }
            throw error(500, `Failed to load list data: ${reason instanceof Error ? reason.message : 'Unknown error'}`);
        }

        // Process groups result (non-critical, form can work without it)
        if (groupsResult.status === 'fulfilled' && groupsResult.value) {
            groupsData = groupsResult.value;
        } else {
            const reason = groupsResult.status === 'rejected' ? groupsResult.reason : new Error('Groups data missing');
            console.error('Edit List page load: Failed to fetch groups:', reason);
            groupsError = `Failed to load groups for sharing options: ${reason instanceof Error ? reason.message : 'Unknown error'}`;
            // Don't throw error here, just pass the message to the component
        }

        return {
            list: listData,
            groups: groupsData,
            error: groupsError // Pass group loading error to the page
        };

    } catch (err) {
        // Catch errors thrown by Promise.allSettled handling or initial setup
        console.error(`Edit List page load: Unexpected error for list ${listId}:`, err);
        // Check if it's a SvelteKit error object before re-throwing
        if (err instanceof Error && 'status' in err && typeof err.status === 'number') {
            throw err;
        }
        throw error(500, `An unexpected error occurred: ${err instanceof Error ? err.message : 'Unknown error'}`);
    }
};