// src/service-worker.ts

/// <reference types="@sveltejs/kit" />
/// <reference lib="webworker" />

// This import IS correct - it's provided by SvelteKit
import { build, files, version } from '$service-worker';

// Declare workbox global if needed (if TS complains after removing @types/workbox-sw)
declare const workbox: any; // Using 'any' for simplicity if specific types cause issues

console.log(`[Service Worker] Version: ${version}`);

// --- Precaching Core Assets ---
// Cache essential SvelteKit build artifacts and static files
workbox.precaching.precacheAndRoute(build);
workbox.precaching.precacheAndRoute(files.map(f => ({ url: f, revision: null })));

// --- Runtime Caching Strategies ---

// Example: Cache Google Fonts (Optional)
workbox.routing.registerRoute(
    ({ url }) => url.origin === 'https://fonts.googleapis.com' || url.origin === 'https://fonts.gstatic.com',
    new workbox.strategies.StaleWhileRevalidate({
        cacheName: 'google-fonts-cache',
        plugins: [
            new workbox.cacheableResponse.CacheableResponse({ statuses: [0, 200] }),
            new workbox.expiration.ExpirationPlugin({ maxEntries: 20, maxAgeSeconds: 30 * 24 * 60 * 60 }), // 30 Days
        ],
    }),
);

// Example: Cache images from origin (Optional)
workbox.routing.registerRoute(
    ({ request, url }) => request.destination === 'image' && url.origin === self.location.origin,
    new workbox.strategies.CacheFirst({
        cacheName: 'images-cache',
        plugins: [
            new workbox.cacheableResponse.CacheableResponse({ statuses: [0, 200] }),
            new workbox.expiration.ExpirationPlugin({
                maxEntries: 50,
                maxAgeSeconds: 30 * 24 * 60 * 60, // 30 Days
                purgeOnQuotaError: true,
            }),
        ],
    }),
);

// --- API GET Request Caching ---

// Cache the list of lists (NetworkFirst: try network, fallback cache)
workbox.routing.registerRoute(
    ({ url, request }) => url.pathname === '/api/v1/lists' && request.method === 'GET',
    new workbox.strategies.NetworkFirst({
        cacheName: 'api-lists-cache',
        plugins: [
            new workbox.cacheableResponse.CacheableResponse({ statuses: [0, 200] }),
            new workbox.expiration.ExpirationPlugin({ maxEntries: 10, maxAgeSeconds: 60 * 60 * 24 }) // 1 Day
        ]
    })
);

// Cache individual list details (NetworkFirst)
workbox.routing.registerRoute(
    ({ url, request }) => /^\/api\/v1\/lists\/\d+$/.test(url.pathname) && request.method === 'GET',
    new workbox.strategies.NetworkFirst({
        cacheName: 'api-list-detail-cache',
        plugins: [
            new workbox.cacheableResponse.CacheableResponse({ statuses: [0, 200] }),
            new workbox.expiration.ExpirationPlugin({ maxEntries: 30, maxAgeSeconds: 60 * 60 * 24 }) // 1 Day
        ]
    })
);

// Cache list status (StaleWhileRevalidate: serve cache fast, update background)
workbox.routing.registerRoute(
    ({ url, request }) => /^\/api\/v1\/lists\/\d+\/status$/.test(url.pathname) && request.method === 'GET',
    new workbox.strategies.StaleWhileRevalidate({
        cacheName: 'api-list-status-cache',
        plugins: [
            new workbox.cacheableResponse.CacheableResponse({ statuses: [0, 200] }),
            new workbox.expiration.ExpirationPlugin({ maxEntries: 30, maxAgeSeconds: 60 * 60 }) // 1 Hour
        ]
    })
);

// --- Service Worker Lifecycle ---

self.addEventListener('install', (event) => {
    console.log('[Service Worker] Install event');
    // Force activation immediately (use with caution, ensure clients handle updates)
    // event.waitUntil(self.skipWaiting());
});

self.addEventListener('activate', (event) => {
    const extendableEvent = event as ExtendableEvent; // Cast for type safety
    console.log('[Service Worker] Activate event');
    // Remove outdated caches managed by Workbox's precaching
    extendableEvent.waitUntil(workbox.precaching.cleanupOutdatedCaches());
    // Take control of uncontrolled clients immediately
    // extendableEvent.waitUntil(self.clients.claim());
});

self.addEventListener('fetch', (event) => {
    // Workbox's registered routes handle fetch events automatically.
    // console.log(`[Service Worker] Fetching: ${event.request.url}`);

    // --- Background Sync Interception (More Advanced - requires Workbox Background Sync module) ---
    // This is a basic example; a full implementation requires the workbox-background-sync package
    // and more robust error handling and queue management.
    // const isApiMutation = event.request.url.startsWith(self.location.origin + '/api/') && event.request.method !== 'GET';
    // if (isApiMutation) {
    //     // Example using BackgroundSync Queue (conceptual)
    //     // Requires: import { Queue } from 'workbox-background-sync';
    //     // const queue = new Queue('apiSyncQueue');
    //     // event.respondWith(
    //     //     fetch(event.request.clone()).catch(() => {
    //     //         console.log('[Service Worker] Fetch failed, queueing request:', event.request.url);
    //     //         return queue.pushRequest({ request: event.request });
    //     //     })
    //     // );
    // }
});

// Optional: Listen for messages from clients (e.g., trigger sync manually)
self.addEventListener('message', (event) => {
    if (event.data && event.data.type === 'TRIGGER_SYNC') {
        console.log('[Service Worker] Received TRIGGER_SYNC message.');
        // Import and call processSyncQueue if structured that way,
        // or use Background Sync's replayRequests.
        // Example: Triggering background sync replay (if using Workbox Background Sync)
        // const queue = new workbox.backgroundSync.Queue('apiSyncQueue');
        // event.waitUntil(queue.replayRequests());
    }
});